/*
    Pezzonoid - An implementation of the classic Arkanoid using the Allegro
    library (version 4) and the Allegro Wrapper
    Copyright (C) 2013, Fabio Pezzo

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "Arkanoid.h"
#include <sstream>


Arkanoid::Arkanoid() 
    : Allegro(640, 480, 24, false, false), //640*480, 24 colori, no mouse
      ticks(0),
      gameOver(false),
      level(1),
      levelText("Level: 1", Point(25,30), Color(255,255,255)),
      score(0),
      scoreText("Score: 0", Point(25,10), Color(255,255,255)),
      lifes(3),
      lifesText("Lifes: 3", Point(25,20), Color(255,255,255)),
      remainingBricks(45),
      backgroundLayer(640, 480),
      middleLayer(640, 480),
      brickLayer(640, 480),
      backgroundSound("back.wav"),
      brickSound("brickSound.wav"),
      barSound("barSound.wav"),
      loseSound("GameOver.wav"),
      victorySound("round_end.wav")
{
  
    add(DrawablePointer(backgroundLayer));
    add(DrawablePointer(middleLayer));
    add(DrawablePointer(brickLayer));
    middleLayer.add(DrawablePointer(bar));
    middleLayer.add(DrawablePointer(ball));


   
    Sprite background("Immagine.bmp", Point(0, 60));
    backgroundLayer.add(background);
    backgroundLayer.add(Sprite("separatorB.bmp",Point(0,50)));
    backgroundLayer.add(Sprite("separatorB.bmp",Point(339,50)));

    backgroundLayer.add(Text("PEZZONOID", Point(615,10), Color(255,255,255), Text::ALIGN_RIGHT));
    backgroundLayer.add(Text("Press ESC to exit", Point(25, 470), Color(255,255,255)));

    backgroundLayer.add(DrawablePointer(scoreText));
    backgroundLayer.add(DrawablePointer(lifesText));
    backgroundLayer.add(DrawablePointer(levelText));

    middleLayer.setNotification(ON_KEY_PRESSED).setNotification(ON_TICK);


for(int i = 1; i < 640; i += 71)
{
 for (int j = 70; j<143; j+= 18) 
  {
  brick.setLeftTopCorner(Point(i,j));
  brickLayer.add(brick); 
  //backgroundLayer.add(DrawablePointer(brick)); 
  }
}
    brickLayer.setNotification(ON_TICK);


    backgroundSound.setLoop(true);
    backgroundSound.play();

}

Arkanoid::~Arkanoid()
{}


void Arkanoid::onTick()
{
  //middleLayer.add(bar);
  Allegro::onTick();
  ++ticks;


  if (getKeyboard().isKeyPressed(KEY_ESC))
  { exit = true;   }



  if ( ball.getLeftTopCorner().getY()+13 == bar.getLeftTopCorner().getY() && ball.getLeftTopCorner().getX() >= bar.getLeftTopCorner().getX() && ball.getLeftTopCorner().getX() <= bar.getLeftTopCorner().getX() + bar.getWidth()) 
  {
        barSound.play();
        ball.vy = -2;
        //determine horizontal direction (which half of pad was hit?)
        if ( ball.getLeftTopCorner().getX()+6 >= bar.getLeftTopCorner().getX()+(bar.getWidth()/2)-8 && ball.getLeftTopCorner().getX()+6 <=  bar.getLeftTopCorner().getX()+(bar.getWidth()/2)+8)
        {ball.vx = 0;}

        if ( ball.getLeftTopCorner().getX()+6 < bar.getLeftTopCorner().getX()+(bar.getWidth()/2)-8 )
        {ball.vx = -2;}

        if ( ball.getLeftTopCorner().getX()+6 > bar.getLeftTopCorner().getX()+(bar.getWidth()/2)+8 )
        {ball.vx = 2;}

       // ball.vx = (ball.getLeftTopCorner().getX() < bar.getLeftTopCorner().getX() + (bar.getWidth()/2) ? -1 : 1);
  }



     if (ball.getLeftTopCorner().getY() >= SCREEN_H - 5 && lifes>0)
  {
    --lifes;
    stringstream ss;
    ss << "Lifes: " << lifes;
    lifesText.setText(ss.str());
  }


  if (lifes == 0 && !gameOver) //GAME OVER
  {     
        gameOver = true;
        backgroundSound.stop();
        middleLayer.unsetNotification(ALL).setNotification(ON_TICK);
        middleLayer.clear();

        middleLayer.add(Text("GAME OVER", Point(640/2, 480/2-80), Color(153,153,153), Text::ALIGN_CENTER, Font("Bauer.tga")));
        loseSound.play();
    
        return;
   }
   
   


  if (remainingBricks == 0  ) //VITTORIA
  {     
        gameOver = true;
        backgroundSound.stop();
        middleLayer.unsetNotification(ALL).setNotification(ON_TICK);
        middleLayer.clear();

        middleLayer.add(Text("YOU WIN!", Point(640/2, 480/2-80), Color(153,153,153), Text::ALIGN_CENTER, Font("Bauer.tga")));
        victorySound.play();

        remainingBricks = 100; //ISTRUZIONE FITTIZIA PER USCIRE DALL'if, IL RETURN NON FUNZIONA
   }


  for(CompositeSprite::iterator i = brickLayer.begin(); i!= brickLayer.end(); i++)
  {
    Brick*brick = dynamic_cast<Brick*>(*i); 
    
   if ( ball.vy < 0 && brick->hitBase(ball.getLeftTopCorner().getX(), ball.getLeftTopCorner().getY()) )
    {
      score+=10; //incrementa lo score
      remainingBricks--; //diminuisco i mattoncini rimanenti
      
      stringstream ss;
      ss << "Score: " << score;
      scoreText.setText(ss.str());
      brickSound.play(); // riproduce il suono di impatto
      
      ball.vy = 2; //cambia Coord pallina
      
    }
    
    if ( ball.vy > 0 && brick->hitUpperSide(ball.getLeftTopCorner().getX(), ball.getLeftTopCorner().getY()) )
    {
      score+=10; //incrementa lo score
      remainingBricks--;
      
      stringstream ss;
      ss << "Score: " << score;
      scoreText.setText(ss.str());
      brickSound.play(); // riproduce il suono di impatto
      
      ball.vy = -2; //cambia Coord pallina OLD: *= -1;
    }
    
    if ( ball.vx > 0 && brick->hitLeftSide(ball.getLeftTopCorner().getX(), ball.getLeftTopCorner().getY()) )
    {
      score+=10; //incrementa lo score
      remainingBricks--;
      
      stringstream ss;
      ss << "Score: " << score;
      scoreText.setText(ss.str());
      brickSound.play(); // riproduce il suono di impatto            
      
      ball.vx = -2; //cambia Coord pallina OLD: *= -1;
    }
    
    if ( ball.vx < 0 && brick->hitRightSide(ball.getLeftTopCorner().getX(), ball.getLeftTopCorner().getY()) )
    {
      score+=10; //incrementa lo score
      remainingBricks--;
      
      stringstream ss;
      ss << "Score: " << score;
      scoreText.setText(ss.str());
      brickSound.play(); // riproduce il suono di impatto      
      
      ball.vx = 2; //cambia Coord pallina OLD: *= -1;
    }
    
  }


  if(ticks % 5 == 0)
  draw();


}





